from uuid import uuid4
from render_polargrid.utils import pyside_cache
from render_polargrid import async_loop
from render_polargrid.polargrid_client import Client

import bpy
import logging


logger = logging.getLogger(__name__)


def redraw(self, context):
    if context.area is None:
        return
    context.area.tag_redraw()

global_polargrid_pools = None

def available_pools(self, context):
    """Returns the list of items used by a polargrid_pool-selector EnumProperty."""
    def pool_display_name(pool):
        pool_name = pool.get('name')
        if pool_name == '':
            pool_name = pool['code']
        num_available_workers = pool.get('num_available_workers')
        if num_available_workers:
            pool_name = "{pool_name} ({num_available_workers}x)".format(
                pool_name=pool_name, num_available_workers=num_available_workers)
        return pool_name

    pools = context.window_manager.polargrid.polargrid_pools
    if not pools:
        return []

    return [(p['code'], pool_display_name(p), p['name']) for p in pools]


class PGWMPropertyGroup(bpy.types.PropertyGroup):
    @classmethod
    def register(cls):
        cls.status = bpy.props.EnumProperty(
            items=[
                ('IDLE', 'IDLE', 'Not doing anything.'),
                ('SAVING', 'SAVING', 'Saving your file.'),
                ('INVESTIGATING', 'INVESTIGATING', 'Finding all dependencies.'),
                ('TRANSFERRING', 'TRANSFERRING', 'Transferring all dependencies.'),
                ('COMMUNICATING', 'COMMUNICATING', 'Communicating with Flamenco Server.'),
                ('DONE', 'DONE', 'Not doing anything, but doing something earlier.'),
                ('ABORTING', 'ABORTING', 'User requested we stop doing something.'),
                ('ABORTED', 'ABORTED', 'We stopped doing something.'),
            ],
            name='Status',
            default='IDLE',
            description='Current status of the Polargrid add-on',
            update=redraw)

        cls.description = bpy.props.StringProperty(
            name='Description',
            default='',
            description='Textual description of what Polargrid is doing',
            update=redraw)

        cls.progress = bpy.props.IntProperty(
            name='Progress',
            default=0,
            description='File transfer progress',
            subtype='PERCENTAGE',
            min=0,
            max=100,
            update=redraw)

        cls.pool = bpy.props.EnumProperty(
            items=available_pools,
            name='Server pools',
            description='Which Server pool to use for jobs',
        )

    # List of managers is stored in 'available_managers' ID property,
    # because I don't know how to store a variable list of strings in a proper RNA property.
    @property
    def polargrid_pools(self) -> list:
        global global_polargrid_pools
        return global_polargrid_pools if global_polargrid_pools is not None else []

    @polargrid_pools.setter
    def polargrid_pools(self, new_polargrid_pools):
        global global_polargrid_pools
        global_polargrid_pools = new_polargrid_pools

    def _get_pool(self):
        for pool in self.polargrid_pools:
            if pool['code'] == self.pool:
                return pool


class PGFetchPoolsOperator(async_loop.AsyncModalOperatorMixin,
                           bpy.types.Operator):
    """Fetches the Polargrid pools available to the user"""
    bl_idname = 'polargrid.fetch_pools'
    bl_label = 'Fetch Available Pools'

    stop_upon_exception = True
    log = logging.getLogger(__name__)

    async def async_execute(self, context):
        logger.info("Start fetching pools")
        client = Client(context, self)
        pools = client.list_server_pools()
        logger.info(f"Received {len(pools)} pools")
        for pool in pools:
            logger.info(pool)

        context.window_manager.polargrid.polargrid_pools = pools
        self.quit()
